//////////////////////////////////////////////////////////////////////
// C++ template file for ccom.  This tries to represent the best practices (or at least not bad).
// 
// HELPFUL WEB PAGES
// =================
//
// Doxygen: http://www.stack.nl/~dimitri/doxygen/docblocks.html
//     C++: http://www.icce.rug.nl/documents/cplusplus/
//
//////////////////////////////////////////////////////////////////////


// BEGIN TEMPLATE...

/*!

  @file
  @brief FIX: write a brief description here for the whole file

  FIX: write long winded explanation here

  @bug FIX: one bug per bug line
  @bug FIX: one bug per bug line
  @todo FIX: add one todo item per todo tag
  @todo FIX: add one todo item per todo tag

  @todo FIX: THESE ARE THE REAL TODO ITEMS FOR THIS FILE...
  @todo FIX: Add example of using slogcxx to track execution
*/


// C language headers that have c++ versions
#include <cassert>	// Assert contract conditions in the code
#include <cstdlib>	// EXIT_SUCCESS/EXIT_FAILURE
#include <cstdio>	// The pain of printf
#include <cerrno>
#include <cmath>
#include <climits>
#include <ctime>
#include <cstring>	// Danger Will Robinson

// C headers without c++ versions (/usr/include)
#include <unistd.h>

// C++ headers
#include <iostream>
#include <sstream>	// string streams.  Do no use strstream
#include <iomanip>	// Formatting for cout and cerr
#include <fstream>	// File streams
#include <typeinfo>	// RTTI - run time type info

// C++ STL - standard template library
#include <string>
#include <vector>	// vectors good
#include <list>
#include <algorithm>		// template algorithms, sort, etc.

// Local headers
#include "template.h"


using namespace std;  // std:: everywhere will drive you nuts


/***************************************************************************
 * LOCAL MACROS and DEFINES
 ***************************************************************************/

// Try to keep macros to a minumum.  
// prefer const variables to #defines

#ifdef __GNUC__
#define UNUSED __attribute((__unused__))
#else
/*!
  \brief GNU CC attribute to denote unused paramters in function calls.

  The attribute remove compiler warning for unused arguments and variable.  Only works
  for GNU compilers such as gcc and g++.

  http://gcc.gnu.org/onlinedocs/gcc-3.4.2/gcc/Variable-Attributes.html
*/
#define UNUSED // FIX: is there a MS way to do this?
#endif


/***************************************************************************
 * MAIN
 ***************************************************************************/

/// \brief Program entry point
///
/// Use EXIT_SUCCESS or EXIT_FAILURE to indicate return code unless
/// the number means something
int
main (UNUSED int argc, UNUSED char *argv[])
{
    bool ok=true;

    cout << "Goes to std out.  \\n does not flush\n"
	 << "Goes to std out.  endl causes stdout to flush" << endl;

    PRINT_CHECKPOINT();

    cerr << "Goes to std err" << endl;
    cout << "compile date: " << __DATE__ << endl;
    cout << "compile time: " << __TIME__ << endl;

#if defined (__linux__)
    cout << "os = linux" << endl;
#elif defined (__APPLE__)
    cout << "os = Apple Mac OSX" << endl;
#elif defined (__win32__)
    cout << "os = Microsoft win32" << endl;
#else
    cout << "WE NO SUPPORT YOUR OS" << endl;
#endif

    PRINT_CHECKPOINT_MSG("Calling some functions and classes");

    cout << "Exiting... " << (ok?"success":"failure") << endl;
    return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}
